# -*- coding: UTF-8 -*-
"""
This is a test module for the Resources.Geometries.Well and WellMarker classes using unittest
"""

import unittest

from GeologicalToolbox.DBHandler import DBHandler
from GeologicalToolbox.Requests import Requests
from GeologicalToolbox.Stratigraphy import StratigraphicObject
from GeologicalToolbox.Wells import WellMarker, Well


class TestWellClass(unittest.TestCase):
    """
    a unittest for Well class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        # self.handler = DBHandler(connection='sqlite:///D:\\data.db', echo=False)
        self.handler = DBHandler(connection='sqlite://', echo=False)
        # self.handler = DBHandler(connection='mysql://stdonn:YWBokMUVYHKoSQ1S@localhost:3306/stdonn')
        self.session = self.handler.get_session()

        # add test data to the database
        self.wells = [
            {
                'name'      : 'Well_1',
                'short_name': 'W1',
                'comment'   : 'A drilled well',
                'east'      : 1234.56,
                'north'     : 123.45,
                'altitude'  : 10.5,
                'depth'     : 555,
                'marker'    : ((10, 'mu', 4, ''),
                               (15, 'so', 3, 'Comment 1'),
                               (16, 'sm', 2, ''),
                               (17, 'su', 1, 'Comment 2'),
                               (5, 'mm', 5, 'Comment 3'))
            }, {
                'name'      : 'Well_2',
                'short_name': 'W2',
                'comment'   : '',
                'east'      : 1000.23,
                'north'     : 2300.34,
                'altitude'  : 342.23,
                'depth'     : 341,
                'marker'    : ((12, 'mo', 6, ''),
                               (120, 'mm', 5, 'Comment 1'),
                               (300, 'Fault', 0, 'Comment 2'),
                               (320, 'mo', 6, ''))
            }, {
                'name'      : 'Well_3',
                'short_name': 'W3',
                'comment'   : 'A third well',
                'east'      : 3454.34,
                'north'     : 2340.22,
                'altitude'  : 342.20,
                'depth'     : 751.13,
                'marker'    : ((34, 'mu', 4, ''),
                               (234, 'so', 3, 'Comment 1'),
                               (345, 'Fault', 0, 'Comment 2'),
                               (635, 'mu', 4, 'Comment 3'),
                               (636, 'Fault', 0, ''),
                               (645, 'mu', 4, ''),
                               (665, 'Fault', 0, ''),
                               (699, 'so', 3, 'Comment 1'))
            }, {
                'name'      : 'Well_4',
                'short_name': 'W4',
                'comment'   : 'A fourth well',
                'east'      : 234,
                'north'     : 5645.45,
                'altitude'  : 234.63,
                'depth'     : 645.21,
                'marker'    : ((34, 'mu', 4, ''),
                               (65, 'so', 3, 'Comment 1'),
                               (70, 'Fault', 0, 'Comment 2'),
                               (85, 'so', 3, 'Comment 1'),
                               (105, 'mu', 4, ''),
                               (123, 'mu', 4, ''),
                               (201, 'so', 3, 'Comment 1'))
            }
        ]

        for well in self.wells:
            new_well = Well(well['name'], well['short_name'], well['depth'], 'spatial_reference_unknown', well['east'],
                            well['north'], well['altitude'], self.session, 'well_set', well['comment'])
            new_well.save_to_db()

            for mark in well['marker']:
                new_well.marker.append(WellMarker(mark[0],
                                                  StratigraphicObject.init_stratigraphy(self.session, mark[1], mark[2],
                                                                                        False),
                                                  self.session, well['name'], mark[3]))
                new_well.marker[-1].save_to_db()

    def test_well_markers_to_thickness(self):
        # type: () -> None
        """
        Tests the Requests.well_markers_to_thickness(...) function

        :return: Nothing
        :raises AssertionError: if a test fails
        """
        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=False,
                                                    use_faulted=False, fault_name='Fault', extent=None)

        self.assertEqual(len(result), 4)
        self.assertTrue(result[0].has_property('thickness'))
        self.assertFalse(result[0].has_property('faulted'))
        self.assertFalse(result[0].has_property('summarised'))
        self.assertEqual(result[0].get_property('thickness').value, 5)
        self.assertEqual(result[1].get_property('thickness').value, 200)
        self.assertEqual(result[2].get_property('thickness').value, 31)
        self.assertEqual(result[3].get_property('thickness').value, 78)
        self.assertEqual(result[0].easting, 1234.56)
        self.assertEqual(result[0].northing, 123.45)
        self.assertEqual(result[0].altitude, 10.5 - 10)
        self.assertEqual(result[0].name, 'Well_1')
        self.assertEqual(result[1].name, 'Well_3')
        self.assertEqual(result[2].name, 'Well_4')
        self.assertEqual(result[3].name, 'Well_4')

        del result

        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=False,
                                                    use_faulted=True, extent=None)
        self.assertEqual(len(result), 5)
        self.assertTrue(result[0].has_property('thickness'))
        self.assertTrue(result[0].has_property('faulted'))
        self.assertFalse(result[0].has_property('summarised'))
        self.assertEqual(result[0].get_property('thickness').value, 5)
        self.assertEqual(result[0].get_property('faulted').value, 0)
        self.assertEqual(result[1].get_property('thickness').value, 200)
        self.assertEqual(result[1].get_property('faulted').value, 0)
        self.assertEqual(result[2].get_property('thickness').value, 54)
        self.assertEqual(result[2].get_property('faulted').value, 1)
        self.assertEqual(result[3].get_property('thickness').value, 31)
        self.assertEqual(result[3].get_property('faulted').value, 0)
        self.assertEqual(result[4].get_property('thickness').value, 78)
        self.assertEqual(result[4].get_property('faulted').value, 0)
        self.assertEqual(result[0].name, 'Well_1')
        self.assertEqual(result[1].easting, 3454.34)
        self.assertEqual(result[1].northing, 2340.22)
        self.assertEqual(result[1].altitude, 342.20 - 34)
        self.assertEqual(result[1].name, 'Well_3')
        self.assertEqual(result[2].altitude, 342.20 - 645)
        self.assertEqual(result[2].name, 'Well_3')
        self.assertEqual(result[3].name, 'Well_4')
        self.assertEqual(result[4].name, 'Well_4')

        del result

        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=True,
                                                    use_faulted=False, fault_name='Fault', extent=None)
        self.assertEqual(len(result), 1)
        self.assertTrue(result[0].has_property('thickness'))
        self.assertFalse(result[0].has_property('faulted'))
        self.assertTrue(result[0].has_property('summarised'))
        self.assertEqual(result[0].get_property('thickness').value, 5)
        self.assertEqual(result[0].get_property('summarised').value, 0)
        self.assertEqual(result[0].easting, 1234.56)
        self.assertEqual(result[0].northing, 123.45)
        self.assertEqual(result[0].altitude, 10.5 - 10)
        self.assertEqual(result[0].name, 'Well_1')

        del result

        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=True,
                                                    use_faulted=True, extent=None)
        self.assertEqual(len(result), 3)
        self.assertTrue(result[0].has_property('thickness'))
        self.assertTrue(result[0].has_property('faulted'))
        self.assertTrue(result[0].has_property('summarised'))
        self.assertEqual(result[0].get_property('thickness').value, 5)
        self.assertEqual(result[0].get_property('faulted').value, 0)
        self.assertEqual(result[0].get_property('summarised').value, 0)
        self.assertEqual(result[1].get_property('thickness').value, 665)
        self.assertEqual(result[1].get_property('faulted').value, 1)
        self.assertEqual(result[1].get_property('summarised').value, 1)
        self.assertEqual(result[2].get_property('thickness').value, 167)
        self.assertEqual(result[2].get_property('faulted').value, 1)
        self.assertEqual(result[2].get_property('summarised').value, 1)
        self.assertEqual(result[0].name, 'Well_1')
        self.assertEqual(result[1].name, 'Well_3')
        self.assertEqual(result[2].name, 'Well_4')
        self.assertEqual(result[2].easting, 234)
        self.assertEqual(result[2].northing, 5645.45)
        self.assertEqual(result[2].altitude, 234.63 - 34)

        del result

        extent = [1000, 1500, 0, 3000]  # Well_1 and Well_2, 1 marker from Well_1
        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=True,
                                                    use_faulted=True, extent=extent)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].name, 'Well_1')

        del result

        extent = [1000, 1500, 2000, 3000]  # only Well_2, no marker
        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=True,
                                                    use_faulted=True, extent=extent)
        self.assertEqual(len(result), 0)

        del result

        extent = [0, 500, 5000, 6000]  # only Well_4
        result = Requests.well_markers_to_thickness(self.session, 'mu', 'so', summarise_multiple=False,
                                                    use_faulted=True, extent=extent)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0].name, 'Well_4')
        self.assertTrue(result[0].has_property('thickness'))
        self.assertEqual(result[0].get_property('thickness').value, 31)
        self.assertEqual(result[1].name, 'Well_4')
        self.assertTrue(result[1].has_property('thickness'))
        self.assertEqual(result[1].get_property('thickness').value, 78)

        del result

        # test failures and exceptions

        """
        :raises AttributeError: if marker_1 and marker_2 are equal
        :raises DatabaseException: if the database query results in less than 2 marker of a well_id
        :raises DatabaseRequestException: if an unexpected query result occurs
        :raises FaultException: if a fault is inside the section and use_faulted is False
        :raises TypeError: if session is not an instance of SQLAlchemy session
        :raises ValueError: if a parameter is not compatible with the required type
        """
        self.assertRaises(AttributeError, Requests.well_markers_to_thickness,
                          self.session, 'mu', 'mu', summarise_multiple=False, use_faulted=True, extent=None)
        self.assertRaises(TypeError, Requests.well_markers_to_thickness,
                          'wrong type', 'mu', 'so', summarise_multiple=False, use_faulted=True, extent=None)
        self.assertRaises(TypeError, Requests.well_markers_to_thickness,
                          self.session, 'mu', 'so', summarise_multiple=False, use_faulted=True, extent='ab')
        self.assertRaises(ValueError, Requests.well_markers_to_thickness,
                          self.session, 'mu', 'so', summarise_multiple=False, use_faulted=True, extent=[1,2,3])
        self.assertRaises(ValueError, Requests.well_markers_to_thickness,
                          self.session, 'mu', 'so', summarise_multiple=False, use_faulted=True, extent=[1, 2, 3, 'ab'])

    def tearDown(self):
        # type: () -> None
        """
        Close session after testruns

        :return: Nothing
        """
        self.handler.close_last_session()


if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
This module provides basic geometries (points and lines) for storing geological data in database.
"""

import sqlalchemy as sq
from sqlalchemy.ext.orderinglist import ordering_list
from sqlalchemy.orm import relationship
from sqlalchemy.orm.session import Session
from typing import List

from GeologicalToolbox.DBHandler import Base, AbstractDBObject
from GeologicalToolbox.GeoObject import AbstractGeoObject
from GeologicalToolbox.Properties import Property
from GeologicalToolbox.Stratigraphy import StratigraphicObject
from GeologicalToolbox.Constants import float_precision


class GeoPoint(Base, AbstractGeoObject):
    """
    Class GeoPoint

    Represents a geological point feature, e.g. for storing line nodes or geological outcrops in a database.
    """
    # define db table name and columns
    __tablename__ = 'geopoints'

    id = sq.Column(sq.INTEGER, sq.Sequence('geopoints_id_seq'), primary_key=True)
    has_z = sq.Column(sq.BOOLEAN, default=False)
    horizon_id = sq.Column(sq.INTEGER, sq.ForeignKey('stratigraphy.id'), default=-1)

    # define relationship to stratigraphic table
    hor = relationship("StratigraphicObject")

    line_id = sq.Column(sq.INTEGER, sq.ForeignKey('lines.id'), default=-1)
    line_pos = sq.Column(sq.INTEGER, default=-1)

    # make the points unique -> coordinates + horizon + belongs to one line?
    sq.UniqueConstraint(AbstractGeoObject.east, AbstractGeoObject.north, AbstractGeoObject.alt, horizon_id, line_id,
                        line_pos, name='u_point_constraint')
    sq.Index('geopoint_coordinate_index', AbstractGeoObject.east, AbstractGeoObject.north)

    # add Property relation
    # order by property name
    properties = relationship("Property", order_by=Property.prop_name, backref="point",
                              primaryjoin='GeoPoint.id==Property.point_id', cascade="all, delete, delete-orphan")

    def __init__(self, horizon, has_z, *args, **kwargs):
        # type: (StratigraphicObject, bool, *object, **object) -> None
        """
        Creates a new GeoPoint

        :param horizon: stratigraphy of the point
        :type horizon: StratigraphicObject
        :param has_z: Is altitude stored as z value?
        :type has_z: bool
        :param args: parameters for AbstractGeoObject initialisation
        :type args: List()
        :param kwargs: parameters for AbstractGeoObject initialisation
        :type kwargs: Dict()
        :return: Nothing
        :raises ValueError: if has_z is not compatible to type bool
        """

        self.has_z = bool(has_z)
        self.horizon = horizon

        # call base class constructor
        AbstractGeoObject.__init__(self, *args, **kwargs)

    def __repr__(self):
        # type: () -> str
        """
        Returns a text-representation of the point

        :return: Returns a text-representation of the point
        :rtype: str
        """
        text = "<GeoPoint(id='{}', horizon='{}', line={}, line-position={})>\n".\
            format(self.id, str(self.horizon), self.line_id, self.line_pos)
        text += AbstractGeoObject.__repr__(self)
        text += "Properties: {}".format(self.properties)
        return text

    def __str__(self):
        # type: () -> str
        """
        Returns a text-representation of the point

        :return: Returns a text-representation of the point
        :rtype: str
        """
        text = "[{}] {} - {} - {}\n".format(self.id, str(self.horizon), self.line_id, self.line_pos)
        text += "AbstractGeoObject: {}\n".format(AbstractGeoObject.__str__(self))
        text += "Properties"
        if len(self.properties) == 0:
            text += "\n  --- None ---"
        else:
            for prop in self.properties:
                text += "\n  " + str(prop)

        return text

    @property
    def horizon(self):
        # type: () -> StratigraphicObject
        """
        The stratigraphy of the point object

        :type: StratigraphicObject or None
        :raises TypeError: if value is not of type Stratigraphy
        """
        return self.hor

    @horizon.setter
    def horizon(self, value):
        # type: (StratigraphicObject) -> None
        """
        see getter
        """
        if (value is not None) and (type(value) is not StratigraphicObject):
            raise TypeError('type of commited value ({}) is not StratigraphicObject!'.format(type(value)))

        if value is None:
            self.hor = None
            self.horizon_id = -1
        else:
            self.hor = value

    # delete z-dimension and information from point
    def del_z(self):
        # type: () -> None
        """
        removes the z-value from the point

        :return: Nothing
        """
        self.alt = 0
        self.has_z = False

    def use_z(self):
        # type: () -> None
        """
        uses the altitude value as z coordinate

        :return: Nothing
        """
        self.has_z = True

    # add and remove property information form point
    def add_property(self, prop):
        # type: (Property) -> None
        """
        Adds a new property to the point

        :param prop: new point property
        :type prop: Property
        :return: Nothing
        :raises TypeError: if log is not of type Property
        """
        if type(prop) is not Property:
            raise TypeError('property {} is not of type Property!'.format(str(prop)))

        self.properties.append(prop)

    def delete_property(self, prop):
        # type: (Property) -> None
        """
        Deletes a property from the point

        :param prop: property to delete
        :type prop: Property

        :return: Nothing
        :raises TypeError: if log is not of type Property
        :raises ValueError: if log is not part of self.properties
        """
        if type(prop) is not Property:
            raise TypeError('property {} is not of type Property!'.format(str(prop)))

        try:
            self.properties.remove(prop)
        except ValueError as e:
            raise ValueError(str(e) + '\nProperty with ID ' + str(prop.id) + ' not found in list!')

    def has_property(self, property_name):
        # type: (str) -> bool
        """
        Returns True, if a property with the name property_name exists for the GeoPoint, else False

        :param property_name: name of the requested property
        :type property_name: str
        :return: Returns True, if a property with the name property_name exists for the GeoPoint, else False
        :rtype: bool
        """
        for prop in self.properties:
            if prop.property_name == property_name:
                return True
        return False

    def get_property(self, property_name):
        # type: (str) -> Property
        """
        Returns requested property, if the property with the name property_name exists for the GeoPoint

        :param property_name: name of the requested property
        :type property_name: str
        :return: Returns the property
        :rtype: Property
        :raises ValueError: if no property with the name exists
        """
        property_name = str(property_name)
        for prop in self.properties:
            if prop.property_name == property_name:
                return prop
        raise ValueError("No property with the name '{}' exists!".format(property_name))

    # overwrite loading method
    @classmethod
    def load_all_without_lines_from_db(cls, session):
        # type: (Session) -> List[Line]
        """
        Returns all points in the database connected to the SQLAlchemy Session session, which are not part of a line.
        This function is similar the AbstractGeoObject.load_all_from_db(...) function.

        :param session: represents the database connection as SQLAlchemy Session
        :type session: Session
        :return: a list of points representing the result of the database query
        :rtype: List[GeoPoint]
        :raises TypeError: if session is not of type SQLAlchemy Session
        """
        if not isinstance(session, Session):
            raise TypeError("'session' is not of type SQLAlchemy Session!")

        result = session.query(GeoPoint). \
            filter(GeoPoint.line_id == -1). \
            order_by(cls.id).all()
        for point in result:
            point.session = session
        return result

    @classmethod
    def load_by_name_without_lines_from_db(cls, name, session):
        # type: (str, Session) -> List[GeoPoint]
        """
        Returns all GeoPoints with the given name in the database connected to the SQLAlchemy Session session, which
        don't belong to a line

        :param name: Only GeoPoints with this name will be returned
        :type name: str
        :param session: represents the database connection as SQLAlchemy Session
        :type session: Session
        :return: a list of GeoPoints representing the result of the database query
        :rtype: List[GeoPoints]
        :raises TypeError: if session is not of type SQLAlchemy Session
        """
        if not isinstance(session, Session):
            raise TypeError("'session' is not of type SQLAlchemy Session!")

        result = session.query(cls).filter(cls.line_id == -1).filter(cls.name_col == name).order_by(cls.id).all()
        for obj in result:
            obj.session = session
        return result

    @classmethod
    def load_in_extent_without_lines_from_db(cls, session, min_easting, max_easting, min_northing, max_northing):
        # type: (Session, float, float, float, float) -> List[Line]
        """
        Returns all points inside the given extent in the database connected to the SQLAlchemy Session session, which
        are not part of a line. This function is similar the AbstractGeoObject.load_in_extent_from_db(...) function.

        :param min_easting: minimal easting of extent
        :type min_easting: float
        :param max_easting: maximal easting of extent
        :type max_easting: float
        :param min_northing: minimal northing of extent
        :type min_northing: float
        :param max_northing: maximal northing of extent
        :type max_northing: float
        :param session: represents the database connection as SQLAlchemy Session
        :type session: Session
        :return: a list of points representing the result of the database query
        :rtype: List[GeoPoint]
        :raises ValueError: if one of the extension values is not compatible to type float
        :raises TypeError: if session is not of type SQLAlchemy Session
        """
        min_easting = float(min_easting)
        max_easting = float(max_easting)
        min_northing = float(min_northing)
        max_northing = float(max_northing)

        if not isinstance(session, Session):
            raise TypeError("'session' is not of type SQLAlchemy Session!")

        result = session.query(GeoPoint).filter(sq.between(GeoPoint.east, min_easting, max_easting)). \
            filter(sq.between(GeoPoint.north, min_northing, max_northing)). \
            filter(GeoPoint.line_id == -1). \
            order_by(cls.id).all()
        for point in result:
            point.session = session
        return result


class Line(Base, AbstractDBObject):
    """
    Class Line

    Represents a geological line feature, e.g. for storing faults or horizon surface expressions.
    """
    # define db table name and columns
    __tablename__ = 'lines'

    id = sq.Column(sq.INTEGER, sq.Sequence('line_id_seq'), primary_key=True)
    closed = sq.Column(sq.BOOLEAN)

    # set stratigraphy of the line
    horizon_id = sq.Column(sq.INTEGER, sq.ForeignKey('stratigraphy.id'))
    hor = relationship("StratigraphicObject")

    # add GeoPoint relation, important is the ordering by line_pos value
    # collection_class function automatically reorders this value in case of insertion or deletion of points
    points = relationship("GeoPoint", order_by=GeoPoint.line_pos, collection_class=ordering_list('line_pos'),
                          backref="line", primaryjoin='Line.id==GeoPoint.line_id',
                          cascade="all, delete, delete-orphan")

    def __init__(self, closed, horizon, points, *args, **kwargs):
        # type: (bool, StratigraphicObject, List[GeoPoint], *object, **object) -> None
        """
        Create a new line.

        :param closed: True if the line should be closed, else False
        :type closed: bool
        :param horizon: Stratigraphy to which the line belongs
        :type horizon: StratigraphicObject
        :param points: list of points which represents the lines nodes
        :type points: List[GeoPoint]
        :param args: parameters for AbstractDBObject initialisation
        :type args: List()
        :param kwargs: parameters for AbstractDBObject initialisation
        :type kwargs: Dict()
        :return: Nothing
        :raises ValueError: Raises ValueError if one of the types cannot be converted
        """
        for pnt in points:
            if type(pnt) is not GeoPoint:
                raise ValueError('At least on point in points is not of type GeoPoint!')

        self.is_closed = bool(closed)
        self.horizon = horizon
        self.points = points

        # check doubled values and stratigraphy in the line
        self.__remove_doubled_points()
        self.__check_line()

        # call base class constructor
        AbstractDBObject.__init__(self, *args, **kwargs)

    def __repr__(self):
        # type: () -> str
        """
        Returns a text-representation of the line

        :return: Returns a text-representation of the line
        :rtype: str
        """
        text = "<Line(id='{}', closed='{}', horizon='{}'\n)>".format(self.id, self.closed, str(self.horizon))
        text += AbstractDBObject.__repr__(self)
        text += "\npoints=\n" + str(self.points)
        return text

    def __str__(self):
        # type: () -> str
        """
        Returns a text-representation of the line

        :return: Returns a text-representation of the line
        :rtype: str
        """
        text = "[{}] {} - {}\n".format(self.id, "closed" if self.closed else "not closed", str(self.horizon))
        text += AbstractDBObject.__str__(self)
        text += "\npoints:\n"

        for point in self.points:
            text += "\n{}".format(str(point))

        return text

    @property
    def is_closed(self):
        # type: () -> bool
        """
        True if the line is closed else False

        :type: bool
        :raises ValueError: if value is not compatible to type bool
        """
        return self.closed

    @is_closed.setter
    def is_closed(self, value):
        # type: (bool) -> None
        """
        see getter
        """
        value = bool(value)
        self.closed = bool(value)

    @property
    def horizon(self):
        # type: () -> StratigraphicObject
        """
        The stratigraphy of the line object

        :type: StratigraphicObject
        :raises TypeError: Raises TypeError if value is not of type Stratigraphy
        """
        return self.hor

    @horizon.setter
    def horizon(self, value):
        # type: (StratigraphicObject) -> None
        """
        see getter
        """
        if (value is not None) and (type(value) is not StratigraphicObject):
            raise TypeError('type of committed value ({}) is not StratigraphicObject!'.format(type(value)))

        if value is None:
            self.hor = None
            self.horizon_id = -1
        else:
            self.hor = value

        # check stratigraphy
        self.__check_line()

    def insert_point(self, point, position):
        # type: (GeoPoint, int) -> None
        """
        Insert a point in the line at the committed index.

        :param point: point to be inserted in the line
        :type point: GeoPoint
        :param position: Index, where points should be inserted in the line
        :type position: int
        :return: Nothing
        :raises TypeError: Raises TypeError if point is not of type GeoPoint
        :raises ValueError: Raises ValueError if position is not of type int or cannot be converted to int
        """
        position = int(position)
        if type(point) is not GeoPoint:
            raise TypeError('point {} is not of type GeoPoint!'.format(str(point)))
        else:
            self.points.insert(position, point)

        # check doubled values and stratigraphy in the line
        self.__remove_doubled_points()
        self.__check_line()

    def insert_points(self, points, position):
        # type: (List[GeoPoint], int) -> None
        """
        Insert a points in the line at the committed index.

        :param points: List of points to be inserted in the line
        :type points: List[GeoPoint]
        :param position: Index, where points should be inserted in the line
        :type position: int
        :return: Nothing
        :raises TypeError: Raises TypeError if one of the points is not of type GeoPoint
        :raises ValueError: Raises ValueError if position is not of type int or cannot be converted to int
        """
        position = int(position)

        for pnt in points:
            if type(pnt) is not GeoPoint:
                raise TypeError('At least on point in points is not of type GeoPoint!')

        # use slicing for points insert
        if len(self.points) <= position:
            self.points[-1:-1] = points
        else:
            self.points[position:position] = points

        # check doubled values and stratigraphy in the line
        self.__remove_doubled_points()
        self.__check_line()

    def get_point_index(self, point):
        # type: (GeoPoint) -> int
        """
        Returns the index of the given point in the line

        :param point: point which has to be looked up
        :type point: GeoPoint
        :return: Index of the point in the line
        :rtype: int
        :raises ValueError: Raises ValueError if committed point is not part of the line
        """
        return self.points.index(point)

    def delete_point(self, point):
        # type: (GeoPoint) -> None
        """
        Deletes the point from the line

        :param point: GeoPoint object which should be deleted
        :type point: GeoPoint
        :return: Nothing
        :raises TypeError: Raises TypeError if point is not of type GeoPoint
        :raises ValueError: Raises ValueError the point is not part of the line
        """
        if type(point) is not GeoPoint:
            raise TypeError('point {} is not of type GeoPoint!'.format(str(point)))

        try:
            self.points.remove(point)
        except ValueError as e:
            raise ValueError(str(e) + '\nGeoPoint with ID ' + str(point.id) + ' not found in list!')

        # check doubled values and stratigraphy in the line
        self.__remove_doubled_points()
        self.__check_line()

    def delete_point_by_coordinates(self, easting, northing, altitude=0):
        # type: (float, float, float) -> None
        """
        Deletes a point with the given coordinates from the line

        :param easting: easting value of the point to be deleted
        :type easting: float
        :param northing: northing value of the point to be deleted
        :type northing: float
        :param altitude: altitude value of the point to be deleted (only necessary if point has z-values!)
        :type altitude: float
        :return: Nothing
        :raises ValueError: Raises ValueError if one parameter is not compatible to type float or no point can be found
        """
        try:
            easting = float(easting)
            northing = float(northing)
            altitude = float(altitude)
        except ValueError:
            raise ValueError('On of the input coordinates is not of type float!')

        # iterate over a copy of the list to avoid iteration issues caused by the deletion of values
        for pnt in self.points[:]:
            if (abs(float(pnt.easting) - easting) < float_precision) and (
                    abs(float(pnt.northing) - northing) < float_precision) and (
                    (abs(float(pnt.altitude) - altitude) < float_precision) or not pnt.has_z):
                self.points.remove(pnt)
                # check doubled values in a line
                self.__remove_doubled_points()
                self.__check_line()
                return

        raise ValueError('Point not found with coordinates {0}/{1}/{2}'.format(easting, northing, altitude))

    def __check_line(self):
        # type: () -> None
        """
        Checks the line for inconsistencies.
        Current checks:
        - stratigraphy check for all points

        :return: Nothing
        """
        for point in self.points:
            point.horizon = self.horizon

    def __remove_doubled_points(self):
        # type: () -> None
        """
        remove points with the same coordinates, if they follow up each other in the points list.

        :return: None
        """
        # create a full copy of the point-list
        tmp_pnts = self.points[:]
        for i in range(len(tmp_pnts) - 1):
            coord_1 = (tmp_pnts[i].easting, tmp_pnts[i].northing, tmp_pnts[i].altitude)
            coord_2 = (tmp_pnts[i + 1].easting, tmp_pnts[i + 1].northing, tmp_pnts[i + 1].altitude)

            # remove doubled points
            if coord_1 == coord_2:
                self.points.remove(tmp_pnts[i + 1])

        # test for first and last point
        # if identical, then remove last point and set closed to True
        coord_1 = (self.points[0].easting, self.points[0].northing, self.points[0].altitude)
        coord_2 = (self.points[-1].easting, self.points[-1].northing, self.points[-1].altitude)

        if coord_1 == coord_2:
            del self.points[-1]
            self.closed = True

        # reorder to ensure right position value without reloading from database
        self.points.reorder()

    @classmethod
    def load_in_extent_from_db(cls, session, min_easting, max_easting, min_northing, max_northing):
        # type: (Session, float, float, float, float) -> List[Line]
        """
        Returns all lines with at least on point inside the given extent in the database connected to the SQLAlchemy
        Session session. This function overloads the AbstractGeoObject.load_in_extent_from_db(...) function.

        :param min_easting: minimal easting of extent
        :type min_easting: float
        :param max_easting: maximal easting of extent
        :type max_easting: float
        :param min_northing: minimal northing of extent
        :type min_northing: float
        :param max_northing: maximal northing of extent
        :type max_northing: float
        :param session: represents the database connection as SQLAlchemy Session
        :type session: Session
        :return: a list of lines representing the result of the database query
        :rtype: List[Line]
        :raises ValueError: if one of the extension values is not compatible to type float
        :raises TypeError: if session is not of type SQLAlchemy Session
        """
        min_easting = float(min_easting)
        max_easting = float(max_easting)
        min_northing = float(min_northing)
        max_northing = float(max_northing)

        if not isinstance(session, Session):
            raise TypeError("'session' is not of type SQLAlchemy Session!")

        # select the points with a line_id that are located inside the extent
        # -> result will be a list of tuples with only a single line-id value
        # -> convert this list to a set in order to remove doubled values

        points = set(
            [x[0] for x in session.query(GeoPoint.line_id).
                filter(GeoPoint.line_id != -1).
                filter(sq.between(GeoPoint.east, min_easting, max_easting)).
                filter(sq.between(GeoPoint.north, min_northing, max_northing)).all()
             ])

        # return line with points in extend
        # to speed up the process, test if points (len > 0) exist in extent first
        if len(points) == 0:
            return []
        result = session.query(Line).filter(Line.id.in_(points)).order_by(cls.id).all()
        for line in result:
            line.session = session
        return result



# -*- coding: UTF-8 -*-

class ArgumentError(Exception):
    """
    This exception should be raised, if a wrong number of arguments are submitted or an other failure inside the
    arguments are recognised.
    """
    def __init__(self, msg):
        self.message = str(msg)

    def __str__(self):
        return self.message


class DatabaseException(Exception):
    """
    This exception should be raised, if an unresolved database issue occurred (e.g. more than one value in a unique
    column
    """

    def __init__(self, msg):
        self.message = str(msg)

    def __str__(self):
        return self.message


class DatabaseRequestException(Exception):
    """
    This exception should be raised, if an error occurs during a database request (e.g. not the expected result)
    """

    def __init__(self, msg):
        self.message = str(msg)

    def __str__(self):
        return self.message


class FaultException(Exception):
    """
    Will be raised, if a fault marker causes an interruption
    """

    def __init__(self, msg):
        self.message = str(msg)

    def __str__(self):
        return self.message


class ListOrderException(Exception):
    """
    Will be raised, if the ordering of a list is wrong (e.g. min values in an extent list after max...)
    """

    def __init__(self, msg):
        self.message = str(msg)

    def __str__(self):
        return self.message


class WellMarkerDepthException(Exception):
    """
    Will be raised, if a new well marker should be located deeper than the depth of the well
    """

    def __init__(self, msg):
        self.message = str(msg)

    def __str__(self):
        return self.message



# -*- coding: UTF-8 -*-
"""
This is a test module for the Resources.Geometries.Well and WellMarker classes using unittest
"""

import math
import unittest

from GeologicalToolbox.Exceptions import WellMarkerDepthException
from GeologicalToolbox.DBHandler import DBHandler
from GeologicalToolbox.Stratigraphy import StratigraphicObject
from GeologicalToolbox.WellLogs import WellLog, WellLogValue
from GeologicalToolbox.Wells import WellMarker, Well
from GeologicalToolbox.Constants import float_precision


class TestWellClass(unittest.TestCase):
    """
    a unittest for Well class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        # self.handler = DBHandler(connection='sqlite:///D:\\data.db', echo=False)
        self.handler = DBHandler(connection='sqlite://', echo=False)
        # self.handler = DBHandler(connection='mysql://stdonn:YWBokMUVYHKoSQ1S@localhost:3306/stdonn')
        self.session = self.handler.get_session()

        # add test data to the database
        self.wells = [
            {
                'name'      : 'Well_1',
                'short_name': 'W1',
                'comment'   : 'A drilled well',
                'east'      : 1234.56,
                'north'     : 123.45,
                'altitude'  : 10.5,
                'depth'     : 555,
                'marker'    : ((10, 'mu', 4, ''),
                               (15, 'so', 3, 'Comment 1'),
                               (16, 'sm', 2, ''),
                               (17, 'su', 1, 'Comment 2'),
                               (5, 'mm', 5, 'Comment 3'))
            }, {
                'name'      : 'Well_2',
                'short_name': 'W2',
                'comment'   : '',
                'east'      : 1000.23,
                'north'     : 2300.34,
                'altitude'  : 342.23,
                'depth'     : 341,
                'marker'    : ((12, 'mo', 6, ''),
                               (120, 'mm', 5, 'Comment 1'),
                               (300, 'Fault', 0, 'Comment 2'),
                               (320, 'mo', 6, ''))
            }, {
                'name'      : 'Well_3',
                'short_name': 'W3',
                'comment'   : 'A third well',
                'east'      : 3454.34,
                'north'     : 2340.22,
                'altitude'  : 342.20,
                'depth'     : 645.21,
                'marker'    : ((34, 'mu', 4, ''),
                               (234, 'so', 3, 'Comment 1'),
                               (345, 'Fault', 0, 'Comment 2'),
                               (635, 'mu', 4, 'Comment 3'))
            }
        ]

        for well in self.wells:
            new_well = Well(well['name'], well['short_name'], well['depth'], 'spatial_reference_unknown', well['east'],
                            well['north'], well['altitude'], self.session, 'well_set', well['comment'])
            new_well.save_to_db()

            for mark in well['marker']:
                new_well.marker.append(WellMarker(mark[0],
                                                  StratigraphicObject.init_stratigraphy(self.session, mark[1], mark[2], False),
                                                  self.session, well['name'], mark[3]))
                new_well.marker[-1].save_to_db()

    def test_init(self):
        # type: () -> None
        """
        Test the initialisation of the database

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        result = self.session.query(Well).all()
        self.assertEqual(len(result), 3, "Wrong number of drill holes ({}). Should be {}".
                         format(len(result), len(self.wells)))
        self.assertEqual(result[1].well_name, 'Well_2')
        self.assertEqual(result[1].short_name, 'W2')
        self.assertEqual(result[2].comment, 'A third well')
        self.assertTrue(math.fabs(float(result[0].easting) - 1234.56) < float_precision)
        self.assertTrue(math.fabs(float(result[1].northing) - 2300.34) < float_precision)
        self.assertTrue(math.fabs(float(result[0].altitude) - 10.5) < float_precision)
        self.assertTrue(math.fabs(float(result[2].depth) - 645.21) < float_precision)
        self.assertEqual(len(result[1].marker), 4)
        self.assertEqual(result[0].marker[2].horizon.name, 'so')
        self.assertEqual(result[2].marker[2].horizon.name, 'Fault')
        self.assertEqual(result[2].marker[2].horizon.age, 0)
        self.assertEqual(result[2].marker[2].comment, 'Comment 2')
        self.assertEqual(result[0].marker[0].horizon.name, 'mm')
        self.assertEqual(result[0].marker[1].horizon.name, 'mu')
        self.assertEqual(result[0].marker[2].horizon.name, 'so')
        self.assertEqual(result[0].marker[3].horizon.name, 'sm')
        self.assertEqual(result[0].marker[4].horizon.name, 'su')

    def test_loading(self):
        # type: () -> None
        """
        Test the different types of loading of lines from the db.
        Part 1: load all wells from the database
        Part 2: load well by name
        Part 3: load wells in given extent
        Part 4: load wells with minimal depth

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        # Part 1: load all wells from the database
        wells = Well.load_all_from_db(self.session)
        self.assertEqual(len(wells), 3)
        self.assertEqual(wells[0].well_name, 'Well_1')
        self.assertEqual(wells[1].well_name, 'Well_2')
        self.assertEqual(wells[2].well_name, 'Well_3')
        self.assertEqual(len(wells[1].marker), 4)
        self.assertTrue(wells[1].marker[0].horizon == wells[1].marker[3].horizon)
        del wells

        # Part 2: load well by name
        well = Well.load_by_wellname_from_db('Well_2', self.session)
        self.assertEqual(well.well_name, 'Well_2')
        self.assertEqual(well.short_name, 'W2')
        self.assertEqual(well.depth, 341)
        self.assertEqual(len(well.marker), 4)
        self.assertEqual(well.marker[1].horizon.name, 'mm')
        self.assertEqual(well.marker[1].depth, 120)
        del well

        # Part 3: load wells in given extent
        # extent x: 500 - 1,300
        # extent y: 0 - 2,400
        # result: 'Well_1' and 'Well_2'
        wells = Well.load_in_extent_from_db(self.session, 500, 1300, 0, 2400)
        self.assertEqual(len(wells), 2)
        self.assertEqual(wells[0].well_name, 'Well_1')
        self.assertEqual(wells[1].well_name, 'Well_2')
        self.assertEqual(len(wells[0].marker), 5)
        self.assertEqual(len(wells[1].marker), 4)
        del wells

        # Part 4: load wells with minimal depth
        wells = Well.load_deeper_than_value_from_db(self.session, 395.23)
        self.assertEqual(len(wells), 2)
        self.assertEqual(wells[0].well_name, 'Well_1')
        self.assertEqual(wells[1].well_name, 'Well_3')
        self.assertTrue(wells[0].depth >= 395.23)
        self.assertTrue(wells[1].depth >= 395.23)
        del wells

    def test_insertion(self):
        # type: () -> None
        """
        Test the insert functions of class Well

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        wells = Well.load_all_from_db(self.session)
        marker_1 = WellMarker(301.43, StratigraphicObject.init_stratigraphy(self.session, "ku"), self.session, 'Comment 1')
        marker_2 = WellMarker(351.65, StratigraphicObject.init_stratigraphy(self.session, "mo"), self.session)
        marker_3 = WellMarker(934.23, StratigraphicObject.init_stratigraphy(self.session, "mm"), self.session, 'Comment 3')

        wells[0].insert_marker(marker_1)
        self.assertEqual(len(wells[0].marker), 6)
        self.assertEqual(wells[0].marker[5], marker_1)
        self.assertRaises(ValueError, wells[1].insert_marker, marker_3)

        wells[2].insert_multiple_marker([marker_1, marker_2])
        self.assertEqual(len(wells[2].marker), 6)
        self.assertEqual(wells[2].marker[2], marker_1)
        self.assertEqual(wells[2].marker[4], marker_2)
        del wells

        wells = Well.load_all_from_db(self.session)
        self.assertEqual(wells[2].marker[2], marker_1)
        self.assertEqual(wells[2].marker[4], marker_2)
        del wells

        wells = Well.load_all_from_db(self.session)
        self.assertRaises(ValueError, wells[1].insert_multiple_marker, [marker_1, marker_2, marker_3])

    def test_get_marker_by_depth(self):
        # type: () -> None
        """
        Test the Well.get_marker_by_depth function

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        well = Well.load_by_wellname_from_db('Well_1', self.session)
        self.assertEqual(well.get_marker_by_depth(16).horizon.name, 'sm')
        self.assertRaises(ValueError, well.get_marker_by_depth, 100)

    def test_delete_marker(self):
        # type: () -> None
        """
        Test the Well.get_marker_by_depth function

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        well = Well.load_by_wellname_from_db('Well_1', self.session)
        marker = well.get_marker_by_depth(16)
        self.assertEqual(marker.horizon.name, 'sm')
        self.assertRaises(ValueError, well.get_marker_by_depth, 100)
        well.delete_marker(marker)
        self.assertEqual(len(well.marker), 4)
        well.save_to_db()
        del well

        well = Well.load_by_wellname_from_db('Well_1', self.session)
        self.assertEqual(len(well.marker), 4)

        new_marker = WellMarker(23423, StratigraphicObject.init_stratigraphy(self.session, 'so'), self.session, 'Nothing')
        self.assertRaises(ValueError, well.delete_marker, new_marker)
        self.assertRaises(TypeError, well.delete_marker, 'test')

    def test_setter_and_getter(self):
        # type: () -> None
        """
        Test setter and getter functionality

        :return: Nothing
        :raises AssertionError: Raises Assertion Error if a test fails
        """
        # first part: test setter
        wells = Well.load_all_from_db(self.session)
        test_string = "abcdefghijklmnopqrstuvwxyz1234567890"
        wells[0].comment = "new comment set"
        wells[1].comment = 4 * test_string
        wells[0].easting = '-344.3'
        wells[1].easting = -1234.34
        # self.assertRaises(ValueError, setattr, wells[2], 'easting', 'test')
        # python >= 2.7 => not included in all ArcGIS versions...
        with(self.assertRaises(ValueError)):  # python <= 2.6
            wells[2].easting = 'test'
        wells[0].northing = -234.56
        wells[1].northing = '-2345.356'
        with(self.assertRaises(ValueError)):
            wells[2].northing = 'test'
        wells[0].altitude = -343.67
        wells[1].altitude = '-235.34'
        with(self.assertRaises(ValueError)):
            wells[2].altitude = 'test'
        wells[0].depth = 235.65
        wells[1].depth = "3456.14"
        with(self.assertRaises(ValueError)):
            wells[2].depth = 'test'
        with(self.assertRaises(ValueError)):
            wells[2].depth = -123.43
        with(self.assertRaises(WellMarkerDepthException)):
            wells[2].depth = 500
        wells[0].well_name = "new Well Name"
        wells[1].well_name = 4 * test_string
        wells[0].short_name = "NWN"
        wells[1].short_name = test_string

        # changes are stored automatically to the database through SQLAlchemy

        del wells

        # second part: test getters
        wells = Well.load_all_from_db(self.session)
        self.assertEqual(wells[0].comment, "new comment set")
        self.assertEqual(len(wells[1].comment), 100)
        self.assertEqual(wells[0].easting, -344.3)
        self.assertEqual(wells[1].easting, -1234.34)
        self.assertEqual(wells[2].easting, 3454.34)
        self.assertEqual(wells[0].northing, -234.56)
        self.assertEqual(wells[1].northing, -2345.356)
        self.assertEqual(wells[2].northing, 2340.22)
        self.assertEqual(wells[0].altitude, -343.67)
        self.assertEqual(wells[1].altitude, -235.34)
        self.assertEqual(wells[2].altitude, 342.2)
        self.assertEqual(wells[0].depth, 235.65)
        self.assertEqual(wells[1].depth, 3456.14)
        self.assertEqual(wells[2].depth, 645.21)
        self.assertEqual(wells[0].well_name, 'new Well Name')
        self.assertEqual(len(wells[1].well_name), 100)
        self.assertEqual(wells[0].short_name, 'NWN')
        self.assertEqual(len(wells[1].short_name), 20)

        # setter and getter for session
        wells[2].session = wells[1].session

    def test_log_handling(self):
        # type: () -> None
        """
        Tests the log handling functionality

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        log_values = (
            (
                (10, 4, '', ''),
                (15, '45.4', 'name 1', 'Comment 1'),
                (16, 34.3, '', ''),
                (17, 234, '', 'Comment 2'),
                (5, '34.4', '', 'Comment 3')
            ),
            (
                (34.3, 4, '', ''),
                (13, 234, '', 'Comment 2'),
                (34, '34.4', '', 'Comment 3')
            )
        )

        well = Well.load_by_wellname_from_db('Well_2', self.session)
        for i in range(len(log_values)):
            log = WellLog('log {}'.format(i), 'unit name', self.session, '', '')
            log.save_to_db()
            well.add_log(log)

            for value in log_values[i]:
                log.insert_log_value(WellLogValue(value[0], value[1], self.session, value[2], value[3]))
            del log

        del well

        well = Well.load_by_wellname_from_db('Well_2', self.session)
        self.assertEqual(2, len(well.logs))
        self.assertEqual(5, len(well.logs[0].log_values))
        self.assertEqual('log 0', well.logs[0].property_name)
        self.assertEqual(3, len(well.logs[1].log_values))
        self.assertEqual('log 1', well.logs[1].property_name)

        well.delete_log(well.logs[1])

        del well

        well = Well.load_by_wellname_from_db('Well_2', self.session)
        self.assertEqual(1, len(well.logs))
        self.assertEqual(5, len(well.logs[0].log_values))
        self.assertEqual('log 0', well.logs[0].property_name)

        logs = WellLog.load_all_from_db(self.session)
        self.assertEqual(1, len(logs))

    def tearDown(self):
        # type: () -> None
        """
        Empty function, nothing to shutdown after the testing process

        :return: Nothing
        """
        pass


class TestWellMarkerClass(unittest.TestCase):
    """
    a unittest for WellMarker class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        self.handler = DBHandler(connection='sqlite://', echo=False)
        self.session = self.handler.get_session()

        # add test data to the database
        self.wells = [
            {
                'name'      : 'Well_1',
                'short_name': 'W1',
                'comment'   : 'A drilled well',
                'east'      : 1234.56,
                'north'     : 123.45,
                'altitude'  : 10.5,
                'depth'     : 555,
                'marker'    : ((10, 'mu', 4, ''),
                               (15, 'so', 3, 'Comment 1'),
                               (16, 'sm', 2, ''),
                               (17, 'su', 1, 'Comment 2'),
                               (5, 'mm', 5, 'Comment 3'))
            }, {
                'name'      : 'Well_2',
                'short_name': 'W2',
                'comment'   : '',
                'east'      : 1000.23,
                'north'     : 2300.34,
                'altitude'  : 342.23,
                'depth'     : 341,
                'marker'    : ((12, 'mo', 6, ''),
                               (120, 'mm', 5, 'Comment 1'),
                               (300, 'Fault', 0, 'Comment 2'),
                               (320, 'mo', 6, ''))
            }, {
                'name'      : 'Well_3',
                'short_name': 'W3',
                'comment'   : 'A third well',
                'east'      : 3454.34,
                'north'     : 2340.22,
                'altitude'  : 342.20,
                'depth'     : 645.21,
                'marker'    : ((34, 'mu', 4, ''),
                               (234, 'so', 3, 'Comment 1'),
                               (345, 'Fault', 0, 'Comment 2'),
                               (635, 'mu', 4, 'Comment 3'))
            }
        ]

        for well in self.wells:
            new_well = Well(well['name'], well['short_name'], well['depth'], 'spatial_reference_unknown', well['east'],
                            well['north'], well['altitude'], self.session, 'well_set', well['comment'])
            new_well.save_to_db()

            for mark in well['marker']:
                new_well.marker.append(WellMarker(mark[0], StratigraphicObject.init_stratigraphy(self.session, mark[1],
                                                                                                 mark[2], False),
                                                  self.session, well['name'], mark[3]))
                new_well.marker[-1].save_to_db()

    def test_WellMarker_init(self):
        # type: () -> None
        """
        Test the initialisation of the database

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        marker = self.session.query(WellMarker).all()
        self.assertEqual(len(marker), 13)
        self.assertEqual(marker[3].horizon.name, 'su')
        self.assertEqual(marker[4].horizon.name, 'mm')

    def test_WellMarker_setter_and_getter(self):
        # type: () -> None
        """
        Test setter and getter functionality

        :return: Nothing
        :raises AssertionError: Raises Assertion Error if a test fails
        """
        marker = WellMarker.load_all_from_db(self.session)
        self.assertEqual(len(marker), 13)
        test_string = "abcdefghijklmnopqrstuvwxyz1234567890"

        # first part: test setter functionality
        marker[0].comment = 'This is a new comment'
        marker[1].comment = 4 * test_string
        marker[2].depth = 123.43
        marker[3].depth = '2345.54'
        with (self.assertRaises(ValueError)):
            marker[4].depth = 'Test'
        marker[4].horizon = StratigraphicObject.init_stratigraphy(self.session, 'z', 50, False)
        with (self.assertRaises(TypeError)):
            marker[5].horizon = 'test'

        # second part: getter functionality
        marker = WellMarker.load_all_from_db(self.session)
        self.assertEqual(marker[0].comment, 'This is a new comment')
        self.assertEqual(len(marker[1].comment), 100)
        self.assertEqual(marker[3].comment, 'Comment 2')
        self.assertEqual(marker[2].depth, 123.43)
        self.assertEqual(marker[1].depth, 15)
        self.assertEqual(marker[4].horizon.name, 'z')
        self.assertEqual(marker[5].horizon.name, 'mo')

    def test_WellMarker_to_GeoPoint(self):
        # type: () -> None
        """
        Test WellMarker.to_GeoPoint functionality

        :return: Nothing
        :raises AssertionError: Raises Assertion Error if a test fails
        """
        marker = WellMarker.load_all_from_db(self.session)[0]
        point = marker.to_geopoint()
        self.assertEqual(point.easting, 1234.56)
        self.assertEqual(point.northing, 123.45)
        self.assertEqual(point.altitude, 0.5)
        self.assertEqual(point.name, 'Well_1')
        self.assertEqual(point.horizon.name, 'mu')

    def tearDown(self):
        # type: () -> None
        """
        Close session after testruns

        :return: Nothing
        """
        self.handler.close_last_session()


if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
This is a test module for the Resources.Geometries.GeoPoint class using unittest
"""

import math
import sys
import unittest

from GeologicalToolbox.DBHandler import DBHandler
from GeologicalToolbox.Geometries import GeoPoint, Line
from GeologicalToolbox.Properties import Property
from GeologicalToolbox.Stratigraphy import StratigraphicObject
from GeologicalToolbox.Constants import float_precision


class TestGeoPointClass(unittest.TestCase):
    """
    This is a unittest class for the Resources.Geometries.GeoPoint class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        # self.handler = DBHandler(connection='sqlite:///C:\\Users\\steph\\PycharmProjects\\Modelling-Toolbox\\tmp.sqlite', echo=False)
        self.handler = DBHandler(connection='sqlite://', echo=False)
        self.session = self.handler.get_session()

        # add test data to the database
        self.points = [
            {
                'coords': (1234134, 5465462, 123),
                'horizon': 'mu',
                'age': 3,
                'name': '',
                'update': False
            },
            {
                'coords': (1254367, 5443636, 156),
                'horizon': 'so',
                'age': 23,
                'name': '',
                'update': False
            },
            {
                'coords': (1265469, 5467929, None),
                'horizon': 'sm',
                'age': 5,
                'name': 'point set',
                'update': False
            },
            {
                'coords': (1273456, 5449672, 101),
                'horizon': 'mu',
                'age': 26,
                'name': 'point set',
                'update': True
            }
        ]

        self.lines = [
            {
                'closed': False,
                'horizon': 'mu',
                'age': 3,
                'update': False,
                'points': ((1204067.0548148106, 634617.5980860253),
                           (1204067.0548148106, 620742.1035724243),
                           (1215167.4504256917, 620742.1035724243),
                           (1215167.4504256917, 634617.5980860253),
                           (1204067.0548148106, 634617.5980860253)),
                'name': 'Line_1'
            }, {
                'closed': True,
                'horizon': 'so',
                'age': 2,
                'update': True,
                'points': ((1179553.6811741155, 647105.5431482664),
                           (1179553.6811741155, 626292.3013778647),
                           (1194354.20865529, 626292.3013778647),
                           (1194354.20865529, 647105.5431482664)),
                'name': 'Line_2'
            }, {
                'closed': False,
                'horizon': 'mm',
                'age': 4,
                'update': True,
                'points': ((1179091.1646903288, 712782.8838459781),
                           (1161053.0218226474, 667456.2684348812),
                           (1214704.933941905, 641092.8288590391),
                           (1228580.428455506, 682719.3123998424),
                           (1218405.0658121984, 721108.1805541387)),
                'name': 'Line_3'
            }, {
                'closed': False,
                'horizon': 'mo',
                'age': 5,
                'update': True,
                'points': ((1149490.1097279799, 691044.6091080031),
                           (1149490.1097279799, 648030.5761158396),
                           (1191579.1097525698, 648030.5761158396),
                           (1149490.1097279799, 648030.5761158396),
                           (1191579.1097525698, 691044.6091080031),
                           (1149490.1097279799, 691044.6091080031)),
                'name': 'Line_2'
            }
        ]

        for point in self.points:
            strat = StratigraphicObject.init_stratigraphy(self.session, point['horizon'], point['age'], point['update'])
            new_point = GeoPoint(strat, False if (point['coords'][2] is None) else True, '', point['coords'][0],
                                 point['coords'][1], 0 if (point['coords'][2] is None) else point['coords'][2],
                                 self.session, point['name'], '')
            new_point.save_to_db()

        for line in self.lines:
            points = list()
            for point in line['points']:
                points.append(GeoPoint(None, False, '', point[0], point[1], 0, self.session, line['name'], ''))
            new_line = Line(line['closed'],
                            StratigraphicObject.init_stratigraphy(self.session, line['horizon'], line['age'],
                                                                  line['update']),
                            points, self.session, line['name'], '')
            new_line.save_to_db()

    def test_init(self):
        # type: () -> None
        """
        Test the initialisation of the database

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """

        pnts = len(self.points)
        for line in self.lines:
            pnts += len(line['points'])

        # 2 points will be automatically deleted and the lines will be closed
        pnts -= 2

        points = self.session.query(GeoPoint)
        count_points = points.count()
        points = points.all()
        stored_horizons = self.session.query(StratigraphicObject).all()
        # for horizon in stored_horizons:
        #	print(str(horizon))

        # print()

        stored_horizons = [x.name for x in stored_horizons]
        # expected number of horizons
        horizons = set([x['horizon'] for x in self.lines] + [x['horizon'] for x in self.points])

        # for point in points:
        #	print(str(point))

        self.assertEqual(count_points, pnts,
                         "Number of points {} doesn't match the number of stored database points {}!". \
                         format(count_points, pnts))

        if sys.version_info[0] == 2:
            self.assertItemsEqual(horizons, stored_horizons, "Horizons doesn't match.\nDatabase: {}\nShould be: {}". \
                                  format(stored_horizons, horizons))
        else:
            self.assertCountEqual(horizons, stored_horizons, "Horizons doesn't match.\nDatabase: {}\nShould be: {}". \
                                  format(stored_horizons, horizons))
        self.assertEqual(points[0].id, 1, "Wrong ID {} for first point. Should be {}". \
                         format(points[0].id, 1))
        self.assertEqual(points[0].horizon.name, 'mu',
                         "Wrong name of stratigraphic unit ({}) of first point. Should be ". \
                         format(points[0].horizon.name, 'mu'))
        self.assertEqual(points[0].horizon.age, 26,
                         "Wrong age for stratigraphic unit ({}) of first point. Should be {}". \
                         format(points[0].horizon.age, 26))

    def test_loading(self):
        # type: () -> None
        """
        Test the loading functions of the GeoPoint class
        /1/ load all from database
        /2/ load by name
        /3/ load in extent

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """

        # /1/ load all from database
        points = GeoPoint.load_all_from_db(self.session)
        pnts_count = len(self.points)
        for line in self.lines:
            pnts_count += len(line['points'])

        # 2 points have been automatically deleted and the lines will be closed
        pnts_count -= 2
        self.assertEqual(len(points), pnts_count,
                         "Wrong point count ({}). Should be {}.".format(len(points), pnts_count))
        self.assertEqual(points[0].id, 1, "First point should have id {}, but has {}.".format(1, points[0].id))
        self.assertTrue(math.fabs(float(points[-1].easting) - 1191579.1097525698) < float_precision,
                        "Wrong easting difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].easting), 1191579.1097525698,
                               math.fabs(float(points[-1].easting) - 1191579.1097525698), float_precision))
        self.assertTrue(math.fabs(float(points[-1].northing) - 691044.6091080031) < float_precision,
                        "Wrong northing difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].northing), 691044.6091080031,
                               math.fabs(float(points[-1].northing) - 691044.6091080031), float_precision))
        del points

        points = GeoPoint.load_all_without_lines_from_db(self.session)
        pnts_count = len(self.points)  # only points which doesn't belong to a line are loaded
        self.assertEqual(len(points), pnts_count,
                         "Wrong point count ({}). Should be {}.".format(len(points), pnts_count))
        self.assertEqual(points[0].id, 1, "First point should have id {}, but has {}.".format(1, points[0].id))
        self.assertTrue(math.fabs(float(points[-1].easting) - 1273456) < float_precision,
                        "Wrong easting difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].easting), 1273456,
                               math.fabs(float(points[-1].easting) - 1273456), float_precision))
        self.assertTrue(math.fabs(float(points[-1].northing) - 5449672) < float_precision,
                        "Wrong northing difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].northing), 5449672,
                               math.fabs(float(points[-1].northing) - 5449672), float_precision))
        self.assertEqual(points[-1].horizon.name, 'mu',
                         "Wrong horizon ({}). Should be {}.".format(points[-1].horizon.name, 'mu'))
        del points

        # /2/ load by name
        points = GeoPoint.load_by_name_from_db('', self.session)

        pnts_count = len(self.points)
        # Added line name as points set name, so we have to comment the line points out...
        # for line in self.lines:
        #    pnts_count += len(line['points'])

        # 2 points will be automatically deleted and the lines will be closed
        # pnts_count -= 2

        # 2 points have another name (obviously they have a name)
        pnts_count -= 2

        self.assertEqual(len(points), pnts_count,
                         "Wrong point count ({}). Should be {}.".format(len(points), pnts_count))
        self.assertEqual(points[0].id, 1, "First point should have id {}, but has {}.".format(1, points[0].id))
        self.assertTrue(math.fabs(float(points[-1].easting) - 1254367) < float_precision,
                        "Wrong easting difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].easting), 1254367,
                               math.fabs(float(points[-1].easting) - 1254367), float_precision))
        self.assertTrue(math.fabs(float(points[-1].northing) - 5443636) < float_precision,
                        "Wrong northing difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].northing), 5443636,
                               math.fabs(float(points[-1].northing) - 5443636), float_precision))
        del points

        points = GeoPoint.load_by_name_without_lines_from_db('', self.session)

        pnts_count = len(self.points)
        # 2 points have another name (obviously they have a name)
        pnts_count -= 2

        self.assertEqual(len(points), pnts_count,
                         "Wrong point count ({}). Should be {}.".format(len(points), pnts_count))
        self.assertEqual(points[0].id, 1, "First point should have id {}, but has {}.".format(1, points[0].id))
        self.assertTrue(math.fabs(float(points[-1].easting) - 1254367) < float_precision,
                        "Wrong easting difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].easting), 1254367,
                               math.fabs(float(points[-1].easting) - 1254367), float_precision))
        self.assertTrue(math.fabs(float(points[-1].northing) - 5443636) < float_precision,
                        "Wrong northing difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[-1].northing), 5443636,
                               math.fabs(float(points[-1].northing) - 5443636), float_precision))
        self.assertEqual(points[-1].horizon.name, 'so',
                         "Wrong horizon ({}). Should be {}.".format(points[-1].horizon.name, 'so'))
        del points

        # /3/ load points in given extent
        # x -> 1174000 - 1200000
        # y ->  613500 -  651000
        # should return points of 2 lines with line-ids 2 (all points) and 4 (1 point)
        points = GeoPoint.load_in_extent_from_db(self.session, 1174000, 1200000, 613500, 651000)

        self.assertEqual(len(points), 5, "Wrong number of points ({}), should be {}".format(len(points), 5))

        self.assertTrue(math.fabs(float(points[0].easting) - 1179553.6811741155) < float_precision,
                        "Wrong easting difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[0].easting), 1179553.6811741155,
                               math.fabs(float(points[0].easting) - 1179553.6811741155), float_precision))
        self.assertTrue(math.fabs(float(points[0].northing) - 647105.5431482664) < float_precision,
                        "Wrong northing difference to large ( |{} - {}| = {} > {}).". \
                        format(float(points[0].northing), 647105.5431482664,
                               math.fabs(float(points[0].northing) - 647105.5431482664), float_precision))
        self.assertEqual(points[0].horizon.name, 'so',
                         "Wrong horizon ({}). Should be {}.".format(points[-1].horizon.name, 'so'))

        del points

        points = GeoPoint.load_in_extent_without_lines_from_db(self.session, 0, 1, 0, 1)
        self.assertEqual(len(points), 0, "Wrong number of points ({}), should be {}".format(len(points), 0))

        del points

    def test_setter_and_getter(self):
        # type: () -> None
        """
        Test the setter and getter functions of class GeoPoint

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        # points = GeoPoint.load_all_from_db(self.session)
        points = GeoPoint.load_all_without_lines_from_db(self.session)
        self.assertEqual(len(points), len(self.points),
                         "Wrong point length ({}). Should be {}.".format(len(points), len(self.points)))
        points[0].easting = 1
        points[1].northing = 2
        points[2].altitude = 3
        points[2].use_z()
        points[3].horizon = StratigraphicObject.init_stratigraphy(self.session, 'so', 10, False)
        points[0].name = 'point set name'
        points[1].del_z()

        for point in points:
            point.save_to_db()

        del points
        points = GeoPoint.load_all_without_lines_from_db(self.session)
        self.assertEqual(len(points), len(self.points),
                         "Wrong point length ({}). Should be {}.".format(len(points), len(self.points)))
        self.assertEqual(points[0].easting, 1, "Wrong easting value ({}). Should be {}.".format(points[0].easting, 1))
        self.assertEqual(points[1].northing, 2,
                         "Wrong northing value ({}). Should be {}.".format(points[1].northing, 2))
        self.assertEqual(points[2].altitude, 3,
                         "Wrong altitude value ({}). Should be {}.".format(points[2].altitude, 3))
        self.assertTrue(points[2].has_z, "Third point has no z-value...")  # Third point got z-value
        self.assertEqual(points[3].horizon.name, 'so',
                         "Wrong horizon ({}). Should be {}.".format(points[3].horizon.name, 'so'))
        self.assertEqual(points[0].name, 'point set name',
                         "Wrong point name ({}). Should be {}.".format(points[0].name, 'point set name'))
        self.assertFalse(points[1].has_z, "Second point has a z-value...")
        self.assertEqual(points[1].altitude, 0,
                         "Wrong altitude value ({}). Should be {}.".format(points[1].altitude, 0))

    def test_add_and_delete_properties(self):
        # type: () -> None
        """
        Test the add_property and delete_property function

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        point = GeoPoint.load_all_from_db(self.session)[0]
        prop = Property('test prop', 'test unit', self.session)
        point.add_property(prop)
        prop = Property('test prop 2', 'test unit 2', self.session)
        point.add_property(prop)

        self.assertRaises(TypeError, point.add_property, 'string')

        del point
        del prop

        point = GeoPoint.load_all_from_db(self.session)[0]
        self.assertEqual(2, len(point.properties))
        self.assertEqual('test prop', point.properties[0].property_name)
        self.assertEqual('test prop 2', point.properties[1].property_name)
        self.assertEqual('test unit', point.properties[0].property_unit)
        self.assertEqual('test unit 2', point.properties[1].property_unit)
        self.assertTrue(point.has_property('test prop 2'))
        self.assertEqual('test unit 2', point.get_property('test prop 2').property_unit)

        prop = point.properties[0]
        point.delete_property(prop)
        self.assertRaises(TypeError, point.delete_property, 'string')
        self.assertRaises(ValueError, point.delete_property, prop)
        self.assertEqual(1, len(point.properties))
        self.assertEqual('test prop 2', point.properties[0].property_name)
        self.assertEqual('test unit 2', point.properties[0].property_unit)

        del point

        point = GeoPoint.load_all_from_db(self.session)[0]
        self.assertEqual(1, len(point.properties))
        self.assertEqual('test prop 2', point.properties[0].property_name)
        self.assertEqual('test unit 2', point.properties[0].property_unit)

    def tearDown(self):
        # type: () -> None
        """
        Close session after testruns

        :return: Nothing
        """
        self.handler.close_last_session()


if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
This module provides a class for storing properties of points (and therefore also for lines). Properties for points are
the same as logs, except there is only one value, not a list of values. They are both derived from the AbstractLogClass.

.. todo:: - reformat docstrings, espacially of setter and getter functions
          - check exception types
"""

import sqlalchemy as sq

from GeologicalToolbox.AbstractLog import AbstractLogClass
from GeologicalToolbox.DBHandler import Base, AbstractDBObject


class Property(Base, AbstractLogClass):
    """
    This class represents logging information for wells
    """
    # define db table name and columns
    __tablename__ = 'properties'

    id = sq.Column(sq.INTEGER, sq.Sequence('properties_id_seq'), primary_key=True)
    point_id = sq.Column(sq.INTEGER, sq.ForeignKey('geopoints.id'), default=-1)
    prop_value = sq.Column(sq.FLOAT, default=0)

    def __init__(self, *args, **kwargs):
        # type: (*object, **object) -> None
        """
        Initialise the class

        :param args: parameters for AbstractLogClass initialisation
        :type args: List()

        :param kwargs: parameters for AbstractLogClass initialisation
        :type kwargs: Dict()
        """
        AbstractLogClass.__init__(self, *args, **kwargs)

    def __repr__(self):
        text = "<Property(value={})>\n".format(self.value)
        text += AbstractLogClass.__repr__(self)
        return text

    def __str__(self):
        text = "{} [{}]: {} - ".format(self.property_name, self.property_unit, self.value)
        text += AbstractDBObject.__str__(self)
        return text

    @property
    def value(self):
        # type: () -> float
        """
        Returns the value of the property

        :return: Returns the value of the property
        """
        return float(self.prop_value)

    @value.setter
    def value(self, prop_value):
        # type: (float) -> None
        """
        Sets a new value for the property

        :param prop_value: new value
        :type prop_value: float

        :return: Nothing
        :raises ValueError: Raises ValueError if prop_value is not type float or cannot be converted to it
        """
        self.prop_value = float(prop_value)



# -*- coding: UTF-8 -*-
"""
This is a test module for the Resources.PropertyLogs-Module classes using unittests
"""

import unittest

from GeologicalToolbox.DBHandler import DBHandler
from GeologicalToolbox.Geometries import GeoPoint
from GeologicalToolbox.Properties import Property
from GeologicalToolbox.Stratigraphy import StratigraphicObject
from GeologicalToolbox.WellLogs import WellLog, WellLogValue
from GeologicalToolbox.Wells import Well


class TestWellLogValueClass(unittest.TestCase):
    """
    a unittest for WellLogValue class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        self.handler = DBHandler(connection='sqlite://', echo=False)
        self.session = self.handler.get_session()

        # add test data to the database
        self.log_values = [
            {
                'depth'  : 200,
                'value'  : 123,
                'name'   : 'log name',
                'comment': 'unknown'
            }, {
                'depth'  : 200.324,
                'value'  : 12.5455,
                'name'   : 'log name 2',
                'comment': 'unknown'
            }, {
                'depth'  : '2345.54',
                'value'  : '641.54',
                'name'   : '',
                'comment': ''
            }
        ]

        for value in self.log_values:
            well_log_value = WellLogValue(value['depth'], value['value'], self.session, value['name'], value['comment'])
            well_log_value.save_to_db()

    def test_WellLogValue_init(self):
        # type: () -> None
        """
        Test the initialisation of the database

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        log_values = self.session.query(WellLogValue).all()
        self.assertEqual(len(log_values), 3)
        self.assertEqual(log_values[0].depth, 200)
        self.assertEqual(log_values[1].value, 12.5455)
        self.assertEqual(log_values[1].name, 'log name 2')
        self.assertEqual(log_values[2].depth, 2345.54)
        self.assertEqual(log_values[2].value, 641.54)
        self.assertEqual(log_values[0].comment, 'unknown')

    def test_setter_and_getter(self):
        # type: () -> None
        """
        Tests the setter and getter functionality of the WellLogValue class

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """

        log_values = WellLogValue.load_all_from_db(self.session)
        self.assertEqual(len(log_values), 3)

        self.assertEqual(log_values[0].depth, 200)
        self.assertEqual(log_values[1].value, 12.5455)

        log_values[2].depth = 123.4
        log_values[0].depth = "54356.45"
        with (self.assertRaises(ValueError)):
            log_values[1].depth = "string"

        log_values[0].value = 4345.4
        log_values[1].value = "3245.4"
        with (self.assertRaises(ValueError)):
            log_values[2].value = "string"

        del log_values

        log_values = WellLogValue.load_all_from_db(self.session)

        self.assertEqual(log_values[0].depth, 54356.45)
        self.assertEqual(log_values[1].depth, 200.324)
        self.assertEqual(log_values[2].depth, 123.4)
        self.assertEqual(log_values[0].value, 4345.4)
        self.assertEqual(log_values[1].value, 3245.4)
        self.assertEqual(log_values[2].value, 641.54)

    def tearDown(self):
        # type: () -> None
        """
        Empty function, nothing to shutdown after the testing process

        :return: Nothing
        """
        pass


class TestWellLogClass(unittest.TestCase):
    """
    a unittest for WellLog class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        self.handler = DBHandler(connection='sqlite://', echo=False)
        self.session = self.handler.get_session()

        # add test data to the database
        self.well = {
            'name'      : 'Well_1',
            'short_name': 'W1',
            'reference' : 'none',
            'east'      : 1234.56,
            'north'     : 123.45,
            'altitude'  : 10.5,
            'depth'     : 555,
            'log_values': ((10, 4, '', ''),
                           (15, '45.4', 'name 1', 'Comment 1'),
                           (16, 34.3, '', ''),
                           (17, 234, '', 'Comment 2'),
                           (5, '34.4', '', 'Comment 3'))
        }

        well = Well(self.well['name'], self.well['short_name'], self.well['depth'], self.well['reference'],
                    self.well['east'], self.well['north'], self.well['altitude'], self.session, '', '')
        well.save_to_db()

        logging = WellLog('logging', 'unit name', self.session, '', '')
        logging.save_to_db()

        well.add_log(logging)

        for value in self.well['log_values']:
            logging.insert_log_value(WellLogValue(value[0], value[1], self.session, value[2], value[3]))

    def test_WellLogValue_init(self):
        # type: () -> None
        """
        Test the initialisation of the database

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        logging = self.session.query(WellLog).one()
        self.assertEqual(len(logging.log_values), 5)
        self.assertEqual(logging.log_values[0].depth, 5)
        self.assertEqual(logging.log_values[1].value, 4)
        self.assertEqual(logging.log_values[2].name, 'name 1')
        self.assertEqual(logging.log_values[2].depth, 15)
        self.assertEqual(logging.log_values[2].value, 45.4)
        self.assertEqual(logging.log_values[4].comment, 'Comment 2')

    def test_setter_and_getter(self):
        # type: () -> None
        """
        Tests the setter and getter functionality of the WellLogValue class

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """

        logging = WellLog.load_all_from_db(self.session)
        self.assertEqual(len(logging), 1)
        logging = logging[0]
        self.assertEqual(logging.log_values[0].depth, 5)
        self.assertEqual(logging.log_values[1].value, 4)
        self.assertEqual(logging.log_values[2].name, 'name 1')
        self.assertEqual(logging.log_values[4].comment, 'Comment 2')
        self.assertEqual(logging.property_name, 'logging')
        self.assertEqual(logging.property_unit, 'unit name')

        logging.property_name = 'new log name'
        logging.property_unit = u'km²'

        del logging

        logging = WellLog.load_all_from_db(self.session)[0]
        self.assertEqual(logging.property_name, 'new log name')
        self.assertEqual(logging.property_unit, u'km²')

        longname = 4 * 'abcdefghijklmnopqrstuvwxyz'
        logging.property_name = longname
        logging.property_unit = longname

        del logging

        logging = WellLog.load_all_from_db(self.session)[0]
        self.assertEqual(logging.property_name, longname[:100])
        self.assertEqual(logging.property_unit, longname[:100])

    def test_insert_and_delete_logvalue(self):
        # type: () -> None
        """
        Test insertion and deletion functionality of class WellLog

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        logging = WellLog.load_all_from_db(self.session)[0]
        logging.insert_log_value(WellLogValue(14.3, 1234, self.session, '', ''))
        self.assertRaises(ValueError, logging.insert_log_value, WellLogValue(556, 1234, self.session, '', ''))

        del logging

        logging = WellLog.load_all_from_db(self.session)[0]
        self.assertEqual(6, len(logging.log_values))
        self.assertEqual(14.3, logging.log_values[2].depth)
        self.assertEqual(1234, logging.get_value_by_depth(14.3).value)
        self.assertRaises(ValueError, logging.get_value_by_depth, 13)
        self.assertRaises(ValueError, logging.get_value_by_depth, 'test')
        self.assertRaises(TypeError, logging.insert_log_value, 'abcde')

        logvalues = [
            WellLogValue(123, 4152, self.session, '', ''),
            WellLogValue(156.34, 3456, self.session, '', ''),
            WellLogValue(16.2, 34.43, self.session, '', ''),
            WellLogValue(15.8, '234.2', self.session, '', '')
        ]

        logging.insert_multiple_log_values(logvalues)
        self.assertRaises(TypeError, logging.insert_multiple_log_values, logvalues.append('abcde'))
        logvalues.pop()
        logvalues.append(WellLogValue(1234, 345, self.session, '', ''))
        self.assertRaises(ValueError, logging.insert_multiple_log_values, logvalues)

        del logging

        logging = WellLog.load_all_from_db(self.session)[0]
        self.assertEqual(10, len(logging.log_values))
        self.assertEqual(14.3, logging.log_values[2].depth)
        self.assertEqual(3456, logging.get_value_by_depth(156.34).value)
        self.assertListEqual([5, 10, 14.3, 15, 15.8, 16, 16.2, 17, 123, 156.34], [x.depth for x in logging.log_values])
        self.assertEqual(34.43, logging.log_values[6].value)

        logging.delete_value(logging.get_value_by_depth(16))

        del logging

        logging = WellLog.load_all_from_db(self.session)[0]
        self.assertEqual(9, len(logging.log_values))
        self.assertEqual(14.3, logging.log_values[2].depth)
        self.assertEqual(234, logging.log_values[6].value)
        self.assertRaises(ValueError, logging.get_value_by_depth, 16)
        self.assertListEqual([5, 10, 14.3, 15, 15.8, 16.2, 17, 123, 156.34], [x.depth for x in logging.log_values])

    def tearDown(self):
        # type: () -> None
        """
        Empty function, nothing to shutdown after the testing process

        :return: Nothing
        """
        pass


class TestPropertyClass(unittest.TestCase):
    """
    a unittest for Property class
    """

    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        self.handler = DBHandler(connection='sqlite://', echo=False)
        self.session = self.handler.get_session()

        # add test data to the database
        point = GeoPoint(StratigraphicObject.init_stratigraphy(self.session, 'mu', 1), True, '', 1, 2, 3, self.session, 'test',
                         '')
        point.save_to_db()

        prop = Property('test prop', 'test unit', self.session)
        point.add_property(prop)
        prop = Property('test prop 2', 'test unit 2', self.session)
        point.add_property(prop)

    def test_init(self):
        # type: () -> None
        """
        Test the initialisation of the class

        :return: Nothing
        :raises AssertionError: Raises AssertionError if a test fails
        """
        point = GeoPoint.load_all_from_db(self.session)[0]
        self.assertEqual(2, len(point.properties))
        self.assertEqual('test prop', point.properties[0].property_name)
        self.assertEqual('test prop 2', point.properties[1].property_name)
        self.assertEqual('test unit', point.properties[0].property_unit)
        self.assertEqual('test unit 2', point.properties[1].property_unit)

    def test_setter_and_getter(self):
        # type: () -> None
        """
        Test the setter and getter functionality

        :return: Nothing
        :raises AssertionError: Raises Assertion Error when a test fails
        """
        point = GeoPoint.load_all_from_db(self.session)[0]
        point.properties[0].value = 342.234
        point.properties[1].value = "345.34"
        point.properties[1].name = "some text information"
        point.properties[1].comment = "unused"

        with self.assertRaises(ValueError):
            point.properties[0].value = "string"

        del point

        point = GeoPoint.load_all_from_db(self.session)[0]
        self.assertEqual(342.234, point.properties[0].value)
        self.assertEqual(345.34, point.properties[1].value)
        self.assertEqual('some text information', point.properties[1].name)
        self.assertEqual('unused', point.properties[1].comment)

    def tearDown(self):
        # type: () -> None
        """
        Close session after testruns

        :return: Nothing
        """
        self.handler.close_last_session()


if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
This is a test module for the Resources.StratigraphicObject.StratigraphicObject class using unittest
"""

import unittest

from GeologicalToolbox.DBHandler import DBHandler
from GeologicalToolbox.Stratigraphy import StratigraphicObject


class TestStratigraphyClass(unittest.TestCase):
    def setUp(self):
        # type: () -> None
        """
        Initialise a temporary database connection for all test cases and fill the database with test data

        :return: None
        """
        # initialise a in-memory sqlite database
        self.handler = DBHandler(connection='sqlite://', echo=False)
        self.session = self.handler.get_session()

        units = [
            {
                'name'  : 'mo',
                'age'   : 1,
                'update': False
            },
            {
                'name'  : 'mm',
                'age'   : 2,
                'update': True
            },
            {
                'name'  : 'mu',
                'age'   : 25,
                'update': False
            },
            {
                'name'  : 'so',
                'age'   : 4,
                'update': False
            },
            {
                'name'  : 'mu',
                'age'   : 3,
                'update': True
            },
            {
                'name'  : 'so',
                'age'   : 5,
                'update': False
            }
        ]

        for unit in units:
            new_unit = StratigraphicObject.init_stratigraphy(self.session, unit['name'], unit['age'], unit['update'])
            new_unit.save_to_db()

    def test_init(self):
        # type: () -> None
        """
        Test the initialisation

        :return: Nothing

        :raises AssertionError: Raises AssertionError if a test fails
        """
        result = self.session.query(StratigraphicObject).all()
        self.assertEqual(len(result), 4, "Wrong number of entries ({}). Should be {}.".format(len(result), 4))

    def test_loading(self):
        # type: () -> None
        """
        Test the loading functions of the stratigraphy class

        /1/ load all from database
        /2/ load by name from database
        /3/ load in age range from database

        :return: Nothing

        :raises AssertionError: Raises AssertionError if a test fails
        """

        # /1/ load all from database
        # additionally update value is tested for mu / so unit
        result = {unit.name: unit for unit in StratigraphicObject.load_all_from_db(self.session)}
        self.assertEqual(len(result), 4, "Wrong number of entries ({}). Should be {}.".format(len(result), 4))
        self.assertEqual(result['mu'].age, 3, "Wrong age for mu ({}). Should be {}".format(result['mu'].age, 3))
        self.assertEqual(result['so'].age, 4, "Wrong age for so ({}). Should be {}".format(result['so'].age, 4))

        del result

        # /2/ load by name from database
        result = StratigraphicObject.load_by_name_from_db('mo', self.session)
        self.assertEqual(result.age, 1, "Wrong age for mo ({}). Should be {}".format(result.age, 3))
        del result

        result = StratigraphicObject.load_by_name_from_db('sm', self.session)
        self.assertIsNone(result, 'Result for sm is not None, but should be: {}'.format(str(result)))
        del result

        # /3/ load in age range from database
        result = StratigraphicObject.load_by_age_from_db(2, 3, self.session)
        self.assertEqual(len(result), 2, "Wrong number of query results ({}). Should be {}.".format(len(result), 2))
        self.assertEqual(result[0].name, 'mm',
                         "Wrong name of first horizon ({}). Should be {}.".format(result[0], 'mm'))
        self.assertEqual(result[1].name, 'mu',
                         "Wrong name of second horizon ({}). Should be {}.".format(result[1], 'mu'))
        self.assertEqual(result[1].age, 3, "Wrong age for second horizon ({}). Should be {}.".format(result[1].age, 3))
        del result

        result = StratigraphicObject.load_by_age_from_db(4, 4, self.session)
        self.assertEqual(len(result), 1, "Wrong number of query results ({}). Should be {}.".format(len(result), 1))
        self.assertEqual(result[0].name, 'so',
                         "Wrong name of first horizon ({}). Should be {}.".format(result[0], 'so'))
        del result

        result = StratigraphicObject.load_by_age_from_db(5, 10, self.session)
        self.assertEqual(len(result), 0, "Wrong number of query results ({}). Should be {}.".format(len(result), 0))

    def test_setter_and_getter(self):
        # type: () -> None
        """
        Test setter and getters of class StratigraphicObject

        :return: Nothing

        :raises AssertionError: Raises AssertionError if a test fails
        """
        result = StratigraphicObject.load_by_name_from_db('mo', self.session)
        result.age = 10
        result.save_to_db()
        del result

        result = StratigraphicObject.load_by_name_from_db('mo', self.session)
        self.assertEqual(result.age, 10, "Wrong age for horizon 'mo' ({}). Should be {}.".format(result.age, 10))

        result.name = 'Blubb'
        result.save_to_db()
        del result

        result = StratigraphicObject.load_by_age_from_db(10, 10, self.session)
        self.assertEqual(len(result), 1, "Wrong number of query results ({}). Should be {}.".format(len(result), 1))
        self.assertEqual(result[0].name, 'Blubb',
                         "Wrong name of stratigraphic unit ({}). Should be {}".format(result[0].name, 'Blubb'))

    def tearDown(self):
        # type: () -> None
        """
        Close session after testruns

        :return: Nothing
        """
        self.handler.close_last_session()


if __name__ == '__main__':
    unittest.main()



# -*- coding: UTF-8 -*-
"""
The setuptools base setup module. Necessary for the PyPI upload.
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# load version
from GeologicalToolbox.Constants import project_version
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))
status = 'Development Status :: '
if project_version[3][0] == 'a':
    status += '3 - Alpha'
elif project_version[3][0] == 'b':
    status += '4 - Beta'
elif project_version[3][0] in ('rc', ''):
    status += '5 - Production/Stable'
# elif project_version[1] == '0' and project_version[3][0] == 'rc':
#     status += '6 - Mature'
else:
    status += '1 - Planning'

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    # $ pip install GeologicalToolbox
    #
    # PyPI: https://pypi.org/project/GeologicalToolbox/

    name='GeologicalToolbox',
    version='.'.join(project_version),
    description='A python module for processing and storing geological data.',
    long_description=long_description,  # Optional
    url='https://github.com/stdonn/GeologicalToolbox',  # Optional
    author='Stephan Donndorf',  # Optional
    author_email='stephan@donndorf.info',  # Optional
    license='MIT',

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers

    classifiers=[
        status,
        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Science/Research',

        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        # other classifiers
        'Natural Language :: English',
        'Topic :: Database :: Front-Ends',
        'Topic :: Documentation :: Sphinx',
        'Topic :: Scientific/Engineering :: GIS',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],

    keywords='Geology Modelling SQLAlchemy',  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(exclude=['venv', 'doc']),  # Required

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'SQLAlchemy>=1.2'
    ],  # Optional

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    extras_require={  # Optional
        ':python_version < "3.5"': [
            'typing'
        ],
        'dev': ['Sphinx', 'sphinxjp.themes.basicstrap']
        # 'test': ['coverage'],
    },

    python_requires='>=2.6, !=3.0.*, !=3.1.*, !=3.2.*'
)
